﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace DxWatcher
{
    public partial class BandMap : Form
    {
        public List<DxBand> bands = null;
        string band = ""; // selected band
        List<spot> spots = new List<spot>();
        TimeSpan timeout = new TimeSpan(1, 0, 0);
        Form1 parent;



        public BandMap(Form1 parent)
        {
            this.parent = parent;
            InitializeComponent();
            dxScale1.map = this;
            dxScale1.frCursor += FreqCursor;

           // dxScale1.Freq = 3.55;
           // dxScale1.FreqWindow = 0.23;
           // dxScale1.ReDraw();
        }

        public void FreqCursor(double freq)
        {
            lbMHz.Text = freq.ToString("F04") + " MHz";
        }

        private void BandMap_FormClosing(object sender, FormClosingEventArgs e)
        {
            this.Hide();
            e.Cancel = true;
        }

        private void BandMap_SizeChanged(object sender, EventArgs e)
        {
            dxScale1.ReDraw();
        }

        private void BandMap_Shown(object sender, EventArgs e)
        {
            cbBand.Items.Clear();
            foreach (DxBand band in bands)
                cbBand.Items.Add(band);
            if (bands.Count > 0)
                cbBand.SelectedIndex = 0;
        }

        private void cbBand_SelectedIndexChanged(object sender, EventArgs e)
        {
            dxScale1.Focus();
            DxBand selectedBand = cbBand.SelectedItem as DxBand;
            band = selectedBand.name;
            dxScale1.Freq = selectedBand.freq1;
            dxScale1.FreqWindow = selectedBand.freq2 - selectedBand.freq1;
            spots.Clear();
            parent.UpdateBandMapSpots();
            dxScale1.ReDraw();
        }

        // add spots from current band that are not too old
        public void NewSpot(spot s)
        {
            TimeSpan span = DateTime.Now.ToUniversalTime() - s.utc;
            if ((s.band == band) && (span < timeout))
            {
                bool done = false;
                foreach (spot s2 in spots)
                {
                    if (s2.call == s.call)
                    {
                        if (s.utc > s2.utc) //  remove obsolete spot
                        {
                            spots.Remove(s2);
                            break;
                        } 
                        done = true;
                    }
                }
                if (!done)
                    spots.Add(s);   // callsing not yet in table, add it 
                spots.Sort(SpotFreqComparer);
                Refresh();
            }
        }

        // remove obsolete spots. Called every 1 minute
        public void CleanupSpots()
        {
            bool done = false;
            while (!done)
            {
                done = true;
                foreach (spot s in spots)
                {
                    if (DateTime.Now.ToUniversalTime() - s.utc > timeout)
                    {
                        spots.Remove(s);
                        done = false;
                        break;
                    }
                }
            }
        }

        private int SpotFreqComparer(spot a, spot b)
        {
            if (a.mhz > b.mhz)
                return 1;
            if (a.mhz < b.mhz)
                return -1;
            return 0;
        }

        private void BandMap_Paint(object sender, PaintEventArgs e)
        {
            Font fnt = new Font(FontFamily.GenericSansSerif,8.0f);
            int lastPosn = 0;
            foreach (spot s in spots)
            {

                if ((s.mhz >= dxScale1.Freq) && (s.mhz <= (dxScale1.Freq + dxScale1.FreqWindow)))
                {
                    int posn = (int)((s.mhz - dxScale1.Freq) / dxScale1.FreqWindow * dxScale1.Height) + dxScale1.Top;
                    int posn2 = Math.Max(posn, lastPosn + 15);
                    lastPosn = posn2;
                    e.Graphics.DrawString(s.call, fnt, Brushes.Black, 10.0f, (float)(posn2 - 7));
                    SizeF ssize = e.Graphics.MeasureString(s.call, fnt);
                    e.Graphics.DrawLine(Pens.Black, 12.0f + ssize.Width, posn2, dxScale1.Left, posn); 
                }
            }
        }

    }
}
