﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace DxWatcher
{
    public partial class DxScale : UserControl
    {
        public delegate void frCursor_(double freq);

        public BandMapDlg map = null;
        public bool done = false;

        int pixMarkSpacing = 5; // mark spacing in pixels;
        double markSpacing = 0.1; // freq space betwwen marks in kHz

        int lastYmouse;

        long start; // start freq as multiple of marker unit
        double freqStart = 18.068;
        double freqStartCorr;
        public double FreqStart // MHz
        {
            set
            {
                freqStartCorr = freqStart = value;
            }
            get
            {
                return freqStartCorr;
            }
        }

        double freqWindow = 100.0;
        double freqWindowCorr;  // kHz
        public double FreqWindow    // MHz
        {
            set
            {
                freqWindowCorr = freqWindow = value * 1000.0;

            }
            get
            {
                return freqWindowCorr / 1000.0;
            }
        }

        public void SetScale()
        {
            double _markSpacing = markSpacing;
            int _pixMarkSpacing = pixMarkSpacing;
            double _freqWindowCorr = freqWindowCorr;
            long _start = start;


            double nspan = freqWindow * Height / 500.0; // normalized span kHz (normalized size 500 pixels)
            if (nspan > 1000)    // > 1000 kHz
            {
                markSpacing = 20.0;
            }
            else if (nspan > 500)    // 500-100 kHz
            {
                markSpacing = 10.0;
            }
            else if (nspan > 200)    // 200 - 500 kHz
            {
                markSpacing = 5.0;
            }
            else if (nspan > 100)    // 100-200 kHz
            {
                markSpacing = 2.0;
            }
            else if (nspan > 50) // 50-100 kHz
            {
                markSpacing = 1.0;
            }
            else if (nspan > 20)  // 20-50 kHz
            {
                markSpacing = 0.5;
            }
            else if (nspan > 10)   // 10-20 kHz
            {
                markSpacing = 0.2;
            }
            else
                markSpacing = 0.1;  // < 10 kHz

            pixMarkSpacing = (int)(Height / (freqWindow / markSpacing)); // pixels between marks, must be > minimum, rounded down
            if (pixMarkSpacing < 3)
                pixMarkSpacing = 3;

            freqWindowCorr = (Height * markSpacing) / pixMarkSpacing;

            start = (long)((freqStart * 1000) / markSpacing);  // start freq to be multiply of markSpacing, rounded down
            freqStartCorr = start * markSpacing / 1000.0;

            if ((_markSpacing != markSpacing) || (_pixMarkSpacing != pixMarkSpacing) || (_freqWindowCorr != freqWindowCorr) || (_start != start))
                map.Refresh();
        }

        public DxScale()
        {
            InitializeComponent();
            this.MouseWheel += DxScale_MouseWheel;
        }

        private void DxScale_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
                lastYmouse = e.Y;
        }

        private void DxScale_MouseMove(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
            {
                int move = e.Y - lastYmouse;
                lastYmouse = e.Y;
                if (move != 0)
                {
                    freqStart -= ((move * markSpacing) / pixMarkSpacing) / 1000.0;
                    SetScale();
                }
            }
            double fr = freqStartCorr + (freqWindowCorr * e.Y) / Height / 1000.0;
        }

        private void DxScale_MouseWheel(object sender, MouseEventArgs e)
        {
            double step = 0.9;
            if ((e.Button == MouseButtons.None) && (e.Delta != 0))
            {
                if (e.Delta < 0)
                {
                    step = 1 / step;
                }
                else if (freqWindow < 10.0)
                    step = 1.0;   // no change, minimum reached

                freqStart += freqWindow * (1 - step) / 2000.0;
                freqWindow *= step;
                SetScale();
            }
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            int lw = Width / 3;
            Graphics gr = e.Graphics;
            gr.DrawLine(Pens.Black, 3, 0, 3, Height);
            long fr = start;
            Font fnt = new Font(FontFamily.GenericSansSerif,8.0f);
            for (int i = 0; i < Height; i += pixMarkSpacing)
            {
                if ((fr % 5) == 0)
                {
                    gr.DrawLine(Pens.Red, 0, i, lw, i);
                    string frs = ((double)(fr*markSpacing/1000.0)).ToString();
                    gr.DrawString(frs, fnt, Brushes.Black, (float)lw,(float)(i-8));
                }
                else
                {
                    gr.DrawLine(Pens.Black, 0, i, 2*lw, i);
                }
                fr++;
            }
        }
    }
}
