﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using DxWatcher.Properties;

namespace DxWatcher
{
    public partial class DxTcvrDlg : Form
    {
        const int timeoutMax = 10;

        // saved in settings
        string port;
        int baud;
        string model;

        bool active = false;
        string rxdata = "";
        int ctr = 0;
        int timeout = 0;
        string vfoAs = "";
        string vfoBs = "";
        string status = "";

        public delegate void tcvrFreqChange_();
        public event tcvrFreqChange_ FreqChange;

        public DxTcvrDlg()
        {
            InitializeComponent();
        }

        public void InitializeSettings()
        {
            foreach (object x in cbPort.Items)
            {
                if (x.ToString() == Settings.Default.TcvrPort)
                {
                    cbPort.SelectedItem = x;
                    break;
                }
            }
            if (cbPort.SelectedIndex < 0)
                cbPort.SelectedIndex = 0;

            foreach (object x in cbSpeed.Items)
            {
                if (int.Parse(x.ToString()) == Settings.Default.TcvrBaud)
                {
                    cbSpeed.SelectedItem = x;
                    break;
                }
            }
            if (cbSpeed.SelectedIndex < 0)
                cbSpeed.SelectedIndex = 0;

            foreach (object x in cbModel.Items)
            {
                if (x.ToString() == Settings.Default.TcvrModel)
                {
                    cbModel.SelectedItem = x;
                    break;
                }
            }
            if (cbModel.SelectedIndex < 0)
                cbModel.SelectedIndex = 0;

            cbActivate.Checked = Settings.Default.TcvrActive;

        }

        public void SaveSettings()
        {
            Settings.Default.TcvrPort = port;
            Settings.Default.TcvrBaud = baud;
            Settings.Default.TcvrModel = model;
            Settings.Default.TcvrActive = cbActivate.Checked;
        }

        public double vfoA
        {
            get
            {
                try
                {
                    return double.Parse(tbVfoA.Text) / 1e6;
                }
                catch
                {
                    return 0.0;
                }
            }
            set
            {
                SetVfo(value, false);
            }
        }

        public double vfoB
        {
            get
            {
                try
                {
                    return double.Parse(tbVfoB.Text) / 1e6;
                }
                catch
                {
                    return 0.0;
                }
            }
            set
            {
                SetVfo(value, true);
            }
        }

        void SetVfo(double mhz, bool vfoB)
        {
            if (active)
            {
                string cmd;
                switch (model)
                {
                    case "FT2000":
                        cmd = ((double)(mhz * 1e6)).ToString("00000000");
                        cmd = vfoB ? "FB" + cmd + ";" : "FA" + cmd + ";";
                        serialPort1.Write(cmd);
                        break;
                    case "TS850":
                        cmd = ((double)(mhz * 1e6)).ToString("00000000000");
                        cmd = vfoB ? "FB" + cmd + ";" : "FA" + cmd + ";";
                        serialPort1.Write(cmd);
                        break;
                }
            }
        }

        public bool vfoBselected = false;

        private void cbActivate_CheckedChanged(object sender, EventArgs e)
        {
            if (cbActivate.Checked)
            {
                active = false;
                try
                {
                    port = cbPort.SelectedItem.ToString();
                    baud = int.Parse(cbSpeed.SelectedItem.ToString());
                    model = cbModel.SelectedItem.ToString();
                    active = true;
                }
                catch { status = "Parameter Error"; }
                cbPort.Enabled = false;
                cbSpeed.Enabled = false;
                cbModel.Enabled = false;
            }
            else
            {
                active = false;
                cbPort.Enabled = true;
                cbSpeed.Enabled = true;
                cbModel.Enabled = true;
            }
        }

        private void DxTcvrDlg_FormClosing(object sender, FormClosingEventArgs e)
        {
            Hide();
            e.Cancel = true;
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            if (active)
            {
                tbVfoA.Text = vfoAs;
                tbVfoB.Text = vfoBs;
                lStatus.Text = status;
                lVfoA.BackColor = vfoBselected ? BackColor : Color.White;
                lVfoB.BackColor = vfoBselected ? Color.White : BackColor;

                if (serialPort1.IsOpen)
                {
                    if (++timeout > timeoutMax)
                    {
                        timeout = timeoutMax;
                        status = "Timeout";
                    }

                    int x;
                    switch (model)
                    {
                        case "FT2000":
                            x = ctr++ % 3;
                            switch (x)
                            {
                                case 0:
                                    serialPort1.Write("FA;");
                                    break;
                                case 1:
                                    serialPort1.Write("FB;");
                                    break;
                                case 2:
                                    serialPort1.Write("VS;");
                                    break;
                            }
                            break;
                        case "TS850":
                            x = ctr++ % 3;
                            switch (x)
                            {
                                case 0:
                                    serialPort1.Write("FA;");
                                    break;
                                case 1:
                                    serialPort1.Write("FB;");
                                    break;
                                case 2:
                                    serialPort1.Write("IF;");
                                    break;
                            }
                            break;
                    }
                }
                else
                {
                    try
                    {
                        serialPort1.BaudRate = baud;
                        serialPort1.PortName = port;
                        serialPort1.Open();
                        status = "Port opened";
                    }
                    catch
                    {
                        status = "Port open failure";
                        serialPort1.Close();
                    }
                }
            }
            else
            {
                if (serialPort1.IsOpen)
                {
                    serialPort1.Close();
                    status = "Port closed";
                }
            }
        }

        private void serialPort1_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
        {
            while (serialPort1.BytesToRead > 0)
            {
                rxdata += (char)serialPort1.ReadByte(); // no encoding
            }

  //          if (rxdata.Length > 1000)
  //              rxdata = ""; // protection

            int posn;
            switch (model)
            {
                case "FT2000":
                    posn = rxdata.IndexOf(';');
                    while (posn >= 0)
                    {
                        string tlg = rxdata.Substring(0, posn);
                        rxdata = rxdata.Substring(posn + 1);
                        posn = rxdata.IndexOf(';');

                        try
                        {
                            if (tlg.Length > 2)
                            {
                                string cmd = tlg.Substring(0, 2);
                                switch (cmd)
                                {
                                    case "FA":
                                        vfoAs = tlg.Substring(2);
                                        timeout = 0;
                                        break;
                                    case "FB":
                                        vfoBs = tlg.Substring(2);
                                        timeout = 0;
                                        break;
                                    case "VS":
                                        vfoBselected = tlg[2] == '1';


                                        timeout = 0;
                                        break;
                                }
                            }
                            if (timeout == 0)
                                status = "OK";

                        }
                        catch { status = "Data Error"; }
                    }
                    break;
                case "TS850":
                    posn = rxdata.IndexOf(';');
                    while (posn >= 0)
                    {
                        string tlg = rxdata.Substring(0, posn);
                        rxdata = rxdata.Substring(posn + 1);
                        posn = rxdata.IndexOf(';');

                        try
                        {
                            if (tlg.Length > 2)
                            {
                                string cmd = tlg.Substring(0, 2);
                                switch (cmd)
                                {
                                    case "FA":
                                        vfoAs = tlg.Substring(2);
                                        timeout = 0;
                                        break;
                                    case "FB":
                                        vfoBs = tlg.Substring(2);
                                        timeout = 0;
                                        break;
                                    case "IF":
                                        switch (tlg[30])
                                        {
                                            case '0':
                                                vfoBselected = false;
                                                break;
                                            case '1':
                                                vfoBselected = true;
                                                break;
                                        }
                                        timeout = 0;
                                        break;
                                }
                            }
                            if (timeout == 0)
                                status = "OK";
                        }
                        catch { status = "Data Error"; }
                    }
                    break;
                default:
                    // rxdata = "";
                    break;
            }
        
        }

        private void tbVfo_TextChanged(object sender, EventArgs e)
        {
            if (FreqChange != null)
                FreqChange();
        }

        private void cbModel_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (cbModel.SelectedItem.ToString() == "TS850")  // default speed 4800 Bd
                cbSpeed.SelectedIndex = 2;
        }
    }
}
