﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace DxWatcher
{
    public partial class FiltersDlg : Form
    {
        const string any = "Any";

        string lastDxCont = "EU";
        string lastSpCont = "Any";
        string lastDxPfx = "";
        string lastSpPfx = "";
        string band = any;
        string mode = any;
        List<DxBand> bands = null;
        List<string> modes = null;

        public FiltersDlg()
        {
            InitializeComponent();
            bands = MainForm.inst.GetBands();
            modes = MainForm.inst.GetModes();
            FillComboBand(true);
            FillComboMode(true);
            ShowFilter(0, true);
        }

        // used to fill dialog data at begin and at filter index change
        private void ShowFilter(int index, bool init)
        {
            if (SpotFilter.inst.filters.Count <= 0)
            {
                rbDx1.Enabled = false;
                rbDx2.Enabled = false;
                rbDx3.Enabled = false;
                rbDx4.Enabled = false;
                rbSp1.Enabled = false;
                rbSp2.Enabled = false;
                rbSp3.Enabled = false;
                rbSp4.Enabled = false;
                cbDx.Enabled = false;
                cbSp.Enabled = false;
                txtDx.Enabled = false;
                txtSp.Enabled = false;
                cbBand.Enabled = false;
                cbMode.Enabled = false;
                cbAlarm.Enabled = false;
                butUpdate.Enabled = false;
                butDelete.Enabled = false;
            }
            else
            {
                rbDx1.Enabled = true;
                rbDx2.Enabled = true;
                rbDx3.Enabled = true;
                rbDx4.Enabled = true;
                rbSp1.Enabled = true;
                rbSp2.Enabled = true;
                rbSp3.Enabled = true;
                rbSp4.Enabled = true;
                cbDx.Enabled = true;
                cbSp.Enabled = true;
                txtDx.Enabled = true;
                txtSp.Enabled = true;
                cbBand.Enabled = true;
                cbMode.Enabled = true;
                cbAlarm.Enabled = true;
                butUpdate.Enabled = true;
                butDelete.Enabled = true;
                if ((index >= 0) && (index < SpotFilter.inst.filters.Count))
                {
                    SpotFilter.FilterItem f = SpotFilter.inst.filters[index];

                    lastDxCont = f.dx;
                    lastDxPfx = f.dx;
                    txtDx.Text = f.dx;

                    lastSpCont = f.spotter;
                    lastSpPfx = f.spotter;
                    txtDx.Text = f.dx;

                    band = f.band;
                    FillComboBand(init);

                    mode = f.mode;
                    FillComboMode(init);

                    switch (f.dxFType)
                    {
                        case SpotFilter.filterType.any:
                            rbDx1.Checked = true;
                            break;
                        case SpotFilter.filterType.call:
                            rbDx2.Checked = true;
                            break;
                        case SpotFilter.filterType.country:
                            FillComboCountry(cbDx, lastDxPfx, init);
                            rbDx3.Checked = true;
                            break;
                        case SpotFilter.filterType.continent:
                            FillComboCont(cbDx, lastDxCont, init);
                            rbDx4.Checked = true;
                            break;
                    }

                    switch (f.spotterFType)
                    {
                        case SpotFilter.filterType.any:
                            rbSp1.Checked = true;
                            break;
                        case SpotFilter.filterType.call:
                            rbSp2.Checked = true;
                            break;
                        case SpotFilter.filterType.country:
                            FillComboCountry(cbSp, lastSpPfx, init);
                            rbSp3.Checked = true;
                            break;
                        case SpotFilter.filterType.continent:
                            FillComboCont(cbSp, lastSpCont, init);
                            rbSp4.Checked = true;
                            break;
                    }

                    //txtBand.Text = filters[index].band.ToString();
                   // txtMode.Text = filters[index].mode;
                    cbAlarm.Checked = f.alarm;
                    lbCount.Text = "Total filter count: " + SpotFilter.inst.filters.Count;
                }
            }
        }

            private void numericUpDown1_ValueChanged(object sender, EventArgs e)
            {
                if (filterNr.Value >= SpotFilter.inst.filters.Count)
                {
                    filterNr.Value = SpotFilter.inst.filters.Count > 0 ? SpotFilter.inst.filters.Count - 1 : 0;
                }
                if (filterNr.Value < 0)
                    filterNr.Value = 0;
                ShowFilter((int)filterNr.Value, false);
            }


            private void butNew_Click(object sender, EventArgs e)
            {
                SpotFilter.FilterItem xfilter = new SpotFilter.FilterItem();
                SpotFilter.inst.filters.Add(xfilter);
                filterNr.Value = SpotFilter.inst.filters.Count - 1;
                ShowFilter(SpotFilter.inst.filters.Count - 1, false);
            }


            private void butDelete_Click(object sender, EventArgs e)
            {
                if (filterNr.Value < SpotFilter.inst.filters.Count)
                    SpotFilter.inst.filters.RemoveAt((int)filterNr.Value);
                lbCount.Text = "Number of filters: " + SpotFilter.inst.filters.Count;
                if ((filterNr.Value >= SpotFilter.inst.filters.Count) && (SpotFilter.inst.filters.Count > 0))
                    filterNr.Value = SpotFilter.inst.filters.Count - 1;
                ShowFilter((int)filterNr.Value, false);
            }

            
            private void butUpdate_Click(object sender, EventArgs e)
            {
                int index = (int)filterNr.Value;
                if ((index < SpotFilter.inst.filters.Count) && (SpotFilter.inst.filters.Count > 0))
                {
                    SpotFilter.FilterItem f = SpotFilter.inst.filters[index];

                    DxBand b = cbBand.SelectedItem as DxBand;
                    if (b != null)
                    {
                        f.band = b.name;
                    }
                    f.mode = cbMode.SelectedItem.ToString().Trim();
                    f.alarm = cbAlarm.Checked;

                    if (rbDx2.Checked)
                    {
                        f.dxFType = SpotFilter.filterType.call;
                        f.dx = txtDx.Text;
                    }
                    else if (rbDx3.Checked)
                    {
                        f.dxFType = SpotFilter.filterType.country;
                        DxCountry.country ctry = (DxCountry.country) cbDx.SelectedItem;
                        f.dx = ctry.pfx;
                    }
                    else if (rbDx4.Checked)
                    {
                        f.dxFType = SpotFilter.filterType.continent;
                        f.dx = cbDx.SelectedItem.ToString();
                    }
                    else
                    {
                        f.dxFType = SpotFilter.filterType.any;
                        f.dx = "";
                    }

                    if (rbSp2.Checked)
                    {
                        f.spotterFType = SpotFilter.filterType.call;
                        f.spotter = txtSp.Text;
                    }
                    else if (rbSp3.Checked)
                    {
                        f.spotterFType = SpotFilter.filterType.country;
                        DxCountry.country ctry = (DxCountry.country)cbSp.SelectedItem;
                        f.spotter = ctry.pfx;
                    }
                    else if (rbSp4.Checked)
                    {
                        f.spotterFType = SpotFilter.filterType.continent;
                        f.spotter = cbSp.SelectedItem.ToString();
                    }
                    else
                    {
                        f.spotterFType = SpotFilter.filterType.any;
                        f.spotter = "";
                    }

                }
            }

        private void DxCheckChanged(object sender, EventArgs e)
        {
            RadioButton rb = sender as RadioButton;
            if (rb != null)
            {
                if (rb.Checked)
                {
                    if (rb == rbDx1)
                    {
                        txtDx.Visible = false;
                        cbDx.Visible = false;
                    }
                    else if (rb == rbDx2)
                    {
                        txtDx.Visible = true;
                        cbDx.Visible = false;
                    }
                    else if (rb == rbDx3)
                    {
                        txtDx.Visible = false;
                        cbDx.Visible = true;
                        FillComboCountry(cbDx, lastDxPfx, true);
                    }
                    else
                    {
                        txtDx.Visible = false;
                        cbDx.Visible = true;
                        FillComboCont(cbDx, lastDxCont, true);
                    }
                }
                else
                {
                    if (rb == rbDx3)
                    {
                        DxCountry.country ctry = (DxCountry.country) cbDx.SelectedItem;
                        lastDxPfx = ctry.pfx;
                    }
                    else if (rb == rbDx4)
                    {
                        string cont = cbDx.SelectedItem as string;
                        if (cont != null)
                        {
                            lastDxCont = cont;
                        }
                    }
                }
            }
        }

        private void SpCheckChanged(object sender, EventArgs e)
        {
            RadioButton rb = sender as RadioButton;
            if (rb != null)
            {
                if (rb.Checked)
                {
                    if (rb == rbSp1)
                    {
                        txtSp.Visible = false;
                        cbSp.Visible = false;
                    }
                    else if (rb == rbSp2)
                    {
                        txtSp.Visible = true;
                        cbSp.Visible = false;
                    }
                    else if (rb == rbSp3)
                    {
                        txtSp.Visible = false;
                        cbSp.Visible = true;
                        FillComboCountry(cbSp, lastSpPfx, true);
                    }
                    else
                    {
                        txtSp.Visible = false;
                        cbSp.Visible = true;
                        FillComboCont(cbSp, lastSpCont, true);
                    }
                }
                else
                {
                    if (rb == rbSp3)
                    {
                        DxCountry.country ctry = (DxCountry.country) cbSp.SelectedItem;
                        lastSpPfx = ctry.pfx;
                    }
                    else if (rb == rbSp4)
                    {
                        string cont = cbSp.SelectedItem as string;
                        if (cont != null)
                        {
                            lastSpCont = cont;
                        }
                    }
                }
            }

        }

        void FillComboCountry(ComboBox cb, string lastPfx, bool init)
        {
            if (init)
                cb.Items.Clear();

            foreach (DxCountry.country ctry in DxCountry.inst.countries)
            {
                if (init)
                    cb.Items.Add(ctry);
                if (ctry.pfx == lastPfx)
                {
                    cb.SelectedItem = ctry;
                }
            }
            if ((cb.SelectedIndex < 0) && (cb.Items.Count > 0))
                cb.SelectedIndex = 0;
        }

        void FillComboCont(ComboBox cb, string lastCont, bool init)
        {
            cb.Items.Clear();
            cb.Items.AddRange(new string[] { "AF", "AS", "EU", "NA", "OC", "SA" });
            if ((lastCont != null) && (lastCont != ""))
                foreach (object item in cb.Items)
                {
                    string cont = item as string;
                    if (cont == lastCont)
                    {
                        cb.SelectedItem = item;
                        break;
                    }
                }
            else
                cb.SelectedIndex = 0;
        }

        void FillComboBand(bool init)
        {
            if (init)
            {
                cbBand.Items.Clear();
                DxBand anyband = new DxBand();
                anyband.name = any;
                cbBand.Items.Add(anyband);
            }
            cbBand.SelectedIndex = 0;
            foreach (DxBand bandx in bands)
            {
                if (init)
                    cbBand.Items.Add(bandx);
                if (band == bandx.name)
                    cbBand.SelectedItem = bandx;
            }
        }

        void FillComboMode(bool init)
        {
            if (init)
            {
                cbMode.Items.Clear();
                cbMode.Items.Add(any);
                
            }
            cbMode.SelectedIndex = 0;
            foreach (string m in modes)
            {
                if (init)
                    cbMode.Items.Add(m);
                if (m == mode)
                    cbMode.SelectedItem = m;
            }
        }
    }
}
