﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using DxWatcher.Properties;
using System.Net.Sockets;
using System.Text.RegularExpressions;
using System.Threading;
using System.IO;
using System.Globalization;
using System.Runtime.InteropServices;

namespace DxWatcher
{

    public partial class Form1 : Form
    {
        const string filterFile = "filters.txt";
        const int maxspots = 5000;  // max size of raw spot table

        public Socket soc = null;
        volatile bool go;    // to be passed to connect thread
        volatile bool done; // to be passed from connect thread
        volatile bool oldDone;
        List<DxFilter> filters = new List<DxFilter>();
        TelnetWindow tw;
        BandMap bm;
        static string rxdata = "";
        string pdata = "";
        // expressions used to process cluster data
        Regex[] regs = new Regex[4] {
            new Regex(@"^DX\sde\s([\w|/]+)[:|\s]\s*([\d|\.]+)\s+([\w|/]+).+(\d{4})Z"),
            new Regex(@"^\s+([\d|\.]+)\s+([\w|/]+)\s+(\d+-\w+-\d+\s\d+)Z.+<([\w|/]+)>"), // 18076.6  JW7QIA      18-May-2013 2344Z  IOTA EU-026                  <K6III> 
            new Regex("^xxxx"),
            new Regex("^yyyy") };
        Regex reLogin = new Regex("[l|L]ogin: ");
        bool loginSent = false;
        Regex rePassword = new Regex("[p|P]assword: ");
        bool passwordSent = false;
        bool alarmActive = false;
        CultureInfo culture;

        public Form1()
        {
            culture = new CultureInfo("en-US");
            InitializeComponent();
            tw = new TelnetWindow(this);
            bm = new BandMap(this);
            done = true;
            oldDone = true;
            toolStripStatusLabel1.Text = "Disconnected";
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void bandPlanToolStripMenuItem_Click(object sender, EventArgs e)
        {
            BandPlanDlg dlg = new BandPlanDlg(this);
            dlg.ShowDialog();
            dataSet1.Tables["bandPlan"].WriteXml("bandPlan.xml");
        }

        private void testToolStripMenuItem_Click(object sender, EventArgs e)
        {
            rawSpotTable.Rows.Clear();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            try
            {
                dataSet1.Tables["bandPlan"].ReadXml("bandPlan.xml");
                bm.bands = GetBands();
                LoadFilters(filterFile);
                ReadOldSpots();
                RemoveObsoleteSpots();
                UpdateFilteredSpots();
                if (Settings.Default.Autoconnect)
                    ConnectCluster();
            }
            catch { };
        }

        private void connectToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ConnectDlg dlg = new ConnectDlg();
            dlg.txtAddress.Text = Settings.Default.Server;
            dlg.txtPort.Text = Settings.Default.Port.ToString();
            dlg.txtLogin.Text = Settings.Default.Login;
            dlg.txtPassword.Text = Settings.Default.Password;
            dlg.cbAuto.Checked = Settings.Default.Autoconnect;
            if (dlg.ShowDialog() == DialogResult.OK)
            {
                tw.Show();
                tw.Focus();
                Settings.Default.Server = dlg.txtAddress.Text;
                Settings.Default.Port = int.Parse(dlg.txtPort.Text);
                Settings.Default.Login = dlg.txtLogin.Text;
                Settings.Default.Password = dlg.txtPassword.Text;
                Settings.Default.Autoconnect = dlg.cbAuto.Checked;

                ConnectCluster();
            }
        }

        private void ConnectCluster()
        {
            ThreadStart ts = new ThreadStart(this.ConnectThread);
            Thread thread = new Thread(ts);
            go = true;
            thread.Start();
        }

        public void ConnectThread ()
        {
            done = false;
            oldDone = false;
            byte[] buffer = new byte[1000];

            if (soc != null)    // get rid of old socket
            {
                if (soc.Connected)
                {
                    soc.Shutdown(SocketShutdown.Both);
                }
                soc.Close();
            }

            Settings.Default.Save();
            soc = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
            try
            {

                soc.Connect(Settings.Default.Server, Settings.Default.Port);
                lock (this)
                {
                    rxdata = ">>> Connected <<<\r\n";
                }


                int count = 0;
                do
                {
                    count = soc.Receive(buffer);
                    lock (this)
                    {
                        for (int i = 0; i < count; i++)
                            rxdata += (char)buffer[i];
                    }

                } while (go && (count > 0));

            }
            catch { }
            soc.Close();
            soc = null;
            done = true;
            return;
        }

        private void LoadFilters(string filename)
        {
            //filters.Add(new DxFilter("","17m",""));
            //filters.Add(new DxFilter("^OK", "80m", ""));
            try
            {
                TextReader txtr = new StreamReader(filename);
                string line = txtr.ReadLine();
                while (line != null)
                {
                    string re = line;
                    if ((line = txtr.ReadLine()) != null)
                    {
                        string mode = line;
                        if ((line = txtr.ReadLine()) != null)
                        {
                            string band = line;
                            if ((line = txtr.ReadLine()) != null)
                            {
                                bool alarm = line == "alarm";
                                DxFilter f = new DxFilter(re, band, mode, alarm);
                                filters.Add(f);
                            }
                        }
                    }
                    line = txtr.ReadLine();
                }
                txtr.Close();
            }
            catch { }
        }

        private void SaveFilters(string filename)
        {
            try
            {
                TextWriter txtw = new StreamWriter(filename);
                foreach (DxFilter filter in filters)
                {
                    txtw.WriteLine(filter.re == null ? "":filter.re.ToString());
                    txtw.WriteLine(filter.mode);
                    txtw.WriteLine(filter.band);
                    txtw.WriteLine(filter.alarm ? "alarm" : "none");
                }
                txtw.Close();
            }
            catch { }
        }

        private void SaveSpots()
        {
            try
            {
                rawSpotTable.WriteXml("spots.xml");
            }
            catch { }
        }

        private void ReadOldSpots()
        {
            try
            {
                rawSpotTable.ReadXml("spots.xml");
            }
            catch { }
        }

        private void RemoveObsoleteSpots()
        {
            DateTime timePoint = DateTime.Now - new TimeSpan(1, 0, 0, 0); // one day ago
            go = true;
            while (go)
            {
                go = false;
                foreach (DataRow dr in rawSpotTable.Rows)
                {
                    if ((DateTime)dr["utc"] < timePoint)
                    {
                        rawSpotTable.Rows.Remove(dr);
                        go = true;
                        break;
                    }
                }
            }
        }
        // fill filtered table from raw table using filters
        private void UpdateFilteredSpots()
        {
            spot s;
            DataRow dr;

            filterSpotTable.Clear();

            foreach (DataRow sr in rawSpotTable.Rows)
            {
                s = new spot();
                s.utc = (DateTime)sr["utc"];
                s.call = (string)sr["call"];
                s.spotter = (string)sr["spotter"];
                s.mhz = (double)sr["mhz"];
                s.mode = (string)sr["mode"];
                s.band = (string) sr["band"];

                if (TestMatch(s))
                {
                    dr = filterSpotTable.NewRow();
                    dr["utc"] = s.utc;
                    dr["call"] = s.call;
                    dr["spotter"] = s.spotter;
                    dr["mhz"] = s.mhz;
                    dr["mode"] = s.mode;
                    dr["band"] = s.band;
                    try
                    {
                        filterSpotTable.Rows.Add(dr);
                    }
                    catch { }
                }

            }
            alarmActive = false;
        }

        public void UpdateBandMapSpots()
        {
            foreach (DataRow sr in rawSpotTable.Rows)
            {
                spot s = new spot();
                s.utc = (DateTime)sr["utc"];
                s.call = (string)sr["call"];
                s.spotter = (string)sr["spotter"];
                s.mhz = (double)sr["mhz"];
                s.mode = (string)sr["mode"];
                s.band = (string)sr["band"];

                bm.NewSpot(s);
            }
        }

       // parcing tlg data - line one by one. Create spot
        private void ParseClusterText()
        {
            pdata = pdata.Replace('\r','\n');  // both \r and \n work the same
            int posn = pdata.IndexOf('\n');
            while (posn >= 0)
            {
                string line = "";
                if (posn > 0)
                {
                    line = pdata.Substring(0, posn);
                    for (int i = 0; i<regs.GetLength(0); i++)
                    {
                        Match m = regs[i].Match(line);
                        if (m.Success)
                        {
                            spot s;
                            switch (i)
                            {
                                case 0: // OK0DXI standard spot
                                    s = new spot();
                                    s.call = m.Groups[3].Captures[0].Value;
                                    s.spotter = m.Groups[1].Captures[0].Value;
                                    s.mhz = double.Parse(m.Groups[2].Captures[0].Value,culture) / 1000.0;
                                    string cas1 = m.Groups[4].Captures[0].Value;
                                    cas1 = cas1.Insert(cas1.Length - 2, ":");
                                    s.utc = DateTime.Parse(cas1,culture);
                                    ProcessSpot(s);
                                    break;
                                case 1: // OK0DXI requested spot
                                    s = new spot();
                                    s.call = m.Groups[2].Captures[0].Value;
                                    s.spotter = m.Groups[4].Captures[0].Value;
                                    s.mhz = double.Parse(m.Groups[1].Captures[0].Value,culture) / 1000.0;
                                    string cas2 = m.Groups[3].Captures[0].Value;
                                    cas2 = cas2.Insert(cas2.Length -2, ":"); // must be separator between hour and minute
                                    s.utc = DateTime.Parse(cas2,culture);
                                    ProcessSpot(s);
                                    break;
                                case 2: // Login request
                                    //SendString(Settings.Default.Login);
                                    break;
                                case 3: //Password request
                                    //SendString(Settings.Default.Password);
                                    break;

                            }
                            break;
                        }
                    }
                }
                pdata = pdata.Substring(posn + 1);
                posn = pdata.IndexOf('\n');
            }
        }

        private void SendString(string s)
        {
            if (soc.Connected)
            {
                byte[] buf = new byte[s.Length + 2];

                for (int i = 0; i < s.Length; i++)
                    buf[i] = (byte)s[i];

                buf[s.Length] = 13;
                buf[s.Length + 1] = 10;

                soc.Send(buf);
            }
        }


        // add band and mode info to spot, call filter, add to datagrid
        private void ProcessSpot(spot s)
        {
            passwordSent = loginSent = true;    // no more need to look for

            s.mode = "???";
            s.band = "??";
            foreach (DataRow br in bandPlanTable.Rows)
            {
                if ((s.mhz >= (double)br["mhz_from"]) && (s.mhz <= (double)br["mhz_to"]))
                {
                    s.band = (string)br["band"];
                    s.mode = (string)br["mode"];
                    break;
                }
            }

            DataRow dr = rawSpotTable.NewRow();
            dr["utc"] = s.utc;
            dr["call"] = s.call;
            dr["spotter"] = s.spotter;
            dr["mhz"] = s.mhz;
            dr["mode"] = s.mode;
            dr["band"] = s.band;
            while (rawSpotTable.Rows.Count > maxspots)
                rawSpotTable.Rows.RemoveAt(rawSpotTable.Rows.Count - 1);    // delete oldest spot when backlog too big
            rawSpotTable.Rows.InsertAt(dr,0);


            if (TestMatch(s))
            {
                dr = filterSpotTable.NewRow();
                dr["utc"] = s.utc;
                dr["call"] = s.call;
                dr["spotter"] = s.spotter;
                dr["mhz"] = s.mhz;
                dr["mode"] = s.mode;
                dr["band"] = s.band;
                try
                {
                    filterSpotTable.Rows.InsertAt(dr, 0);
                    if (dataGridView1.Rows.Count > 0)
                        dataGridView1.CurrentCell = dataGridView1.FirstDisplayedCell = dataGridView1.Rows[0].Cells[0];
                }
                catch { }   // spot already exists (conflict on primary key), don't add it to table
            }

            if (bm != null)
                bm.NewSpot(s);
        }

        // call filters one by one, if any matches
        private bool TestMatch(spot s)
        {
            bool retval = false;

            if (filters.Count <= 0)
                return true;        // no filters, all matches

            foreach (DxFilter filter in filters)    // filters can overlap so we need to test all (reason alarm)
            {
                if (filter.TestRule(s, ref alarmActive))
                {
                    retval = true;    // one filter matches, accepted.
                    if (alarmActive)    // no reason to test remaining filters when alarm already active
                        break;
                }
            }
            return retval; // non of filters matches, refused
        }

        private void rawDataToolStripMenuItem_Click(object sender, EventArgs e)
        {
            RawSpotsDlg dlg = new RawSpotsDlg(this);
            dlg.ShowDialog();
        }

        private void filterToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FiltersDlg dlg = new FiltersDlg(filters);
            dlg.ShowDialog();
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            go = false;
            if (soc != null)
            {
                if (soc.Connected)
                    soc.Shutdown(SocketShutdown.Both);
                soc.Close();
            }
            SaveSpots();
            SaveFilters(filterFile);
            while (!done)
                go = false;
        }

        private void terminalWindowToolStripMenuItem_Click(object sender, EventArgs e)
        {
            tw.Show();
            tw.Focus();
        }

        [DllImport("user32.dll")]
        private static extern bool SetForegroundWindow(IntPtr hWnd);

        private void timer1_Tick(object sender, EventArgs e)
        {

            if (rxdata.Length > 0)
            {
                rxdata = rxdata.Replace('\a', ' ');
                lock (this) // process received data here
                {
                    tw.txtRx.Text += rxdata;
                    pdata += rxdata;
                    rxdata = "";
                }

                // it must be here - login and password req is not line terminated by <CR><LF>
                if (!loginSent && (reLogin.IsMatch(pdata)))
                {
                    SendString(Settings.Default.Login);
                    loginSent = true;
                }
                else if (!passwordSent && (rePassword.IsMatch(pdata)))
                {
                    SendString(Settings.Default.Password);
                    passwordSent = true;
                }

                while (tw.txtRx.Text.Length > 10000)
                {
                    tw.txtRx.Text = tw.txtRx.Text.Substring(500);
                }
                tw.txtRx.SelectionStart = tw.txtRx.Text.Length;
                tw.txtRx.SelectionLength = 0;
                tw.txtRx.ScrollToCaret();
                tw.txtRx.Refresh();
            }

            if (alarmActive)
            {
                System.Media.SystemSounds.Beep.Play();
                WindowState = FormWindowState.Normal;
                SetForegroundWindow(this.Handle);
            }

            ParseClusterText();

            if (!oldDone && done)
            {
                loginSent = passwordSent = false;
                rxdata += "\r\n>>> Connection terminated <<<\r\n";
                tw.Focus();
            }
            oldDone = done;

            toolStripStatusLabel1.Text = done ? "Disconnected" : "Connected to " + Settings.Default.Server;
        }

 
        private void tableToolStripMenuItem_Click(object sender, EventArgs e)
        {
            UpdateFilteredSpots();
        }

        private void ackAlarmToolStripMenuItem_Click(object sender, EventArgs e)
        {
            alarmActive = false;
        }

        private void bandMapToolStripMenuItem_Click(object sender, EventArgs e)
        {
            bm.Show();
            bm.Focus();
        }

        public List<DxBand> GetBands()
        {
            List<DxBand> list = new List<DxBand>();

            foreach (DataRow dr in bandPlanTable.Rows)
            {
                string sband = (string)dr["Band"];
                bool found = false;
                foreach (DxBand band in list)
                {
                    if (band.name == sband)
                    {
                        found = true;
                        double f1 = (double)dr["MHz_from"];
                        double f2 = (double)dr["MHz_to"];
                        if (f1 < band.freq1)
                            band.freq1 = f1;
                        if (f2 > band.freq2)
                            band.freq2 = f2;
                    }
                }
                if (!found)
                {
                    DxBand newBand = new DxBand();
                    newBand.name = sband;
                    newBand.freq1 = (double)dr["MHz_from"];
                    newBand.freq2 = (double)dr["MHz_to"];
                    list.Add(newBand);
                }
            }
            return list;
        }
    }

    public struct spot
    {
        public DateTime utc;
        public double mhz;
        public string call;
        public string band;
        public string mode;
        public string spotter;

        public override string ToString()
        {
            return call + " " + band + " " + mode;
        }
    }

    public class DxFilter
    {
        public Regex re = null;  // regular expression filter
        public string band;
        public string mode;
        public bool alarm = false;

        public DxFilter(string expr, string band, string mode, bool alarm)
        {
            if ((expr != null) && (expr.Length > 0))
                re = new Regex(expr);
            this.band = band;
            this.mode = mode.ToUpper();
            this.alarm = alarm;
        }

        public bool TestRule(spot s, ref bool alarmActive)
        {
            if ((re != null) && !re.IsMatch(s.call))
                    return false;

            if ((band != null) && (band.Length > 0) && (s.band != band))
                return false;

            if ((mode != null) && (mode.Length > 0) && (mode != s.mode.ToUpper()))
                return false;

            if (alarm)
                alarmActive = true;
            return true;
        }
    }

    public class DxBand
    {
        public double freq1 = Double.MaxValue;
        public double freq2 = Double.MinValue;
        public string name = "";

        public override string ToString()
        {
            return name;
        }
    }
}
