﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using DxWatcher.Properties;
using System.Net.Sockets;
using System.Text.RegularExpressions;
using System.Threading;
using System.IO;
using System.Globalization;
using System.Runtime.InteropServices;

namespace DxWatcher
{

    public partial class MainForm : Form
    {
        const string filterFile = "filters.xml";
        const int maxspots = 5000;  // max size of raw spot table
        public const int bmrefresh = 120; // removal of obsolete spots refresh cycle [s * 2]

        public volatile Socket soc = null;
        volatile public bool suspend = false;
        volatile public bool start = false;    // set by main program to allow / stop connection
        volatile public bool connected = false; // set by connection thread - actual status
        volatile bool done; // status of connect thread (false = being performed); passed from connect thread
        TelnetWindow tw;
        List<BandMapDlg> bmaps = new List<BandMapDlg>();
        public List<DxBand> bands = null;
        BandMapDlg bm;
        public DxTcvrDlg tcvr;
        static string socketData = "";
        string pdata = "";
        // expressions used to process cluster data
        Regex[] regs = new Regex[4] {
            new Regex(@"^DX\sde\s([\w|/]+)[:|\s]\s*([\d|\.]+)\s+([\w|/]+).+(\d{4})Z"),   // DX de YY5MKI:     7143.4  RZ1AWT       tnx Russia                     2309Z
            new Regex(@"^\s+([\d|\.]+)\s+([\w|/]+)\s+(\d+-\w+-\d+\s\d+)Z.+<([\w|/]+)>"), // 18076.6  JW7QIA      18-May-2013 2344Z  IOTA EU-026                  <K6III> 
            new Regex("^xxxx"),
            new Regex("^yyyy") };
        Regex reLogin = new Regex("[l|L]ogin: ");
        Regex rePassword = new Regex("[p|P]assword: ");
        Regex reCall = new Regex("your call:");
        Regex reCallsign = new Regex("valid callsign:");
        bool loginDone = false;
        bool alarmActive = false;
        CultureInfo culture;
        public int bmCounter = bmrefresh; // to generate 1 minute cycle for obsolete spots cleanup
        public int liveCtr = 120;  // livecounter for RX data, 1 minute
        DxCountry ctry = new DxCountry();
        SpotFilter sfilter = new SpotFilter();
        public static MainForm inst;    // only one instance

        public MainForm()
        {
            start = false;
            inst = this;
            culture = new CultureInfo("en-US");
            InitializeComponent();

            DataGridViewCellStyle style = new DataGridViewCellStyle();
            style.Format = "N4";
            dataGridView1.Columns[1].DefaultCellStyle = style;

            tw = new TelnetWindow(this);
            bm = new BandMapDlg(this);
            tcvr = new DxTcvrDlg();
            tcvr.InitializeSettings();
            toolStripStatusLabel1.Text = "Disconnected";
            tcvr.FreqChange += new DxTcvrDlg.tcvrFreqChange_(tcvr_FreqChange);
        }

        void tcvr_FreqChange()
        {
            bm.Refresh();
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void bandPlanToolStripMenuItem_Click(object sender, EventArgs e)
        {
            BandPlanDlg dlg = new BandPlanDlg(this);
            dlg.ShowDialog();
            dataSet1.Tables["bandPlan"].WriteXml("bandPlan.xml");
        }


        private void Form1_Load(object sender, EventArgs e)
        {
            try
            {
                Width = Settings.Default.Width;
                Height = Settings.Default.Height;
                bandMapAutoToolStripMenuItem.Checked = Settings.Default.BMauto;
                ctry.LoadTables("country.dat");
                dataSet1.Tables["bandPlan"].ReadXml("bandPlan.xml");
                bands = GetBands();
                sfilter.LoadFilters(filterFile);
                ReadOldSpots();
                RemoveObsoleteSpots();
                UpdateFilteredSpots();
                if (bandMapAutoToolStripMenuItem.Checked)
                    bandMapToolStripMenuItem_Click(sender, e);
            }
            catch { };
        }

        private void connectToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ConnectDlg dlg = new ConnectDlg();
            dlg.txtAddress.Text = Settings.Default.Server;
            dlg.txtPort.Text = Settings.Default.Port.ToString();
            dlg.txtLogin.Text = Settings.Default.Login;
            dlg.txtPassword.Text = Settings.Default.Password;
            dlg.cbAuto.Checked = Settings.Default.Autoconnect;
            if (dlg.ShowDialog() == DialogResult.OK)
            {
                tw.Show();
                tw.Focus();
                Settings.Default.Server = dlg.txtAddress.Text;
                Settings.Default.Port = int.Parse(dlg.txtPort.Text);
                Settings.Default.Login = dlg.txtLogin.Text;
                Settings.Default.Password = dlg.txtPassword.Text;
                Settings.Default.Autoconnect = dlg.cbAuto.Checked;

                ConnectCluster();
            }
        }

        public void ConnectCluster()
        {
            start = true;
            ThreadStart ts = new ThreadStart(this.ConnectThread);
            Thread thread = new Thread(ts);
            thread.Start();
        }

        public void ConnectThread ()
        {
            bool oldConnected = false;
            byte[] buffer = new byte[1000];

            if (soc != null)    // get rid of old socket
            {
                if (soc.Connected)
                {
                    soc.Shutdown(SocketShutdown.Both);
                }
                soc.Close();
            }

            soc = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
            try
            {

                soc.Connect(Settings.Default.Server, Settings.Default.Port);
                lock (this)
                {
                    socketData = ">>> Connected <<<\r\n";
                    bmCounter = 0;  // initiate table cleanup asap
                }


                int count = 0;
                connected = true;
                do
                {
                    count = soc.Receive(buffer);
                    lock (this)
                    {
                        for (int i = 0; i < count; i++)
                            socketData += (char)buffer[i];
                    }

                } while (start && (count > 0));
            }
            catch { }
            if (soc != null)
            {
                if (soc.Connected)
                    soc.Disconnect(false);
                soc.Close();
                soc = null;
            }
            lock (this)
            {
                   socketData += "\r\n>>> Connection terminated <<<\r\n";
            }
            connected = false;
            return;
        }

        /// <summary>
        /// Save old spots to XML file
        /// </summary>
        private void SaveSpots()
        {
            try
            {
                rawSpotTable.WriteXml("spots.xml");
            }
            catch { }
        }

        /// <summary>
        /// Read old spots from XML file
        /// </summary>
        private void ReadOldSpots()
        {
            try
            {
                rawSpotTable.ReadXml("spots.xml");
            }
            catch { }
        }

        /// <summary>
        /// Cleanup of more than 1 day old spots in raw table
        /// </summary>
        private void RemoveObsoleteSpots()
        {
            DateTime timePoint = DateTime.Now - new TimeSpan(1, 0, 0, 0); // one day ago
            bool xgo = true;
            while (xgo)
            {
                xgo = false;
                foreach (DataRow dr in rawSpotTable.Rows)
                {
                    if ((DateTime)dr["utc"] < timePoint)
                    {
                        rawSpotTable.Rows.Remove(dr);
                        xgo = true;
                        break;
                    }
                }
            }
        }

        /// <summary>
        /// Fill filtered table from raw table using filters
        /// </summary>
        private void UpdateFilteredSpots()
        {
            spot s;
            DataRow dr;

            filterSpotTable.Clear();

            foreach (DataRow sr in rawSpotTable.Rows)
            {
                s = new spot();
                s.utc = (DateTime)sr["utc"];
                s.call = (string)sr["call"];
                s.spotter = (string)sr["spotter"];
                s.mhz = (double)sr["mhz"];
                s.mode = (string)sr["mode"];
                s.band = (string) sr["band"];

                if (sfilter.TestMatch(s, ref alarmActive, out s.cont))
                {
                    dr = filterSpotTable.NewRow();
                    dr["utc"] = s.utc;
                    dr["call"] = s.call;
                    dr["spotter"] = s.spotter;
                    dr["mhz"] = s.mhz;
                    dr["mode"] = s.mode;
                    dr["band"] = s.band;
                    try
                    {
                        filterSpotTable.Rows.Add(dr);
                    }
                    catch { }
                }

            }
            alarmActive = false;    // cancel possbile alarm
        }

        /// <summary>
        ///  Send all spots to bandmap
        /// </summary>
        public void UpdateBandMapSpots()
        {
            bool doRefresh = false;
            foreach (DataRow sr in rawSpotTable.Rows)
            {
                spot s = new spot();
                s.utc = (DateTime)sr["utc"];
                s.call = (string)sr["call"];
                s.spotter = (string)sr["spotter"];
                s.mhz = (double)sr["mhz"];
                s.mode = (string)sr["mode"];
                s.band = (string)sr["band"];
                s.cont = sr.IsNull("cont")?"":(string)sr["cont"];

                doRefresh |= bm.NewSpot(s);
            }
            if (doRefresh == true)
                bm.Refresh();
        }

       /// <summary>
       /// parcing tlg data - line one by one. Create spot
       /// </summary>
        private void ParseClusterText()
        {
            pdata = pdata.Replace('\r','\n');  // both \r and \n work the same
            int posn = pdata.IndexOf('\n');
            while (posn >= 0)
            {
                string line = "";
                if (posn > 0)
                {
                    line = pdata.Substring(0, posn);
                    for (int i = 0; i<regs.GetLength(0); i++)
                    {
                        Match m = regs[i].Match(line);
                        if (m.Success)
                        {
                            spot s;
                            switch (i)
                            {
                                case 0: // OK0DXI standard spot
                                    s = new spot();
                                    s.call = m.Groups[3].Captures[0].Value;
                                    s.spotter = m.Groups[1].Captures[0].Value;
                                    s.mhz = double.Parse(m.Groups[2].Captures[0].Value,culture) / 1000.0;
                                    string cas1 = m.Groups[4].Captures[0].Value;    // example 2134
                                    s.utc = UtcToDatetime(cas1);
                                    ProcessSpot(s);
                                    break;
                                case 1: // OK0DXI requested spot
                                    s = new spot();
                                    s.call = m.Groups[2].Captures[0].Value;
                                    s.spotter = m.Groups[4].Captures[0].Value;
                                    s.mhz = double.Parse(m.Groups[1].Captures[0].Value,culture) / 1000.0;
                                    string cas2 = m.Groups[3].Captures[0].Value;    // example 
                                    cas2 = cas2.Insert(cas2.Length -2, ":"); // must be separator between hour and minute
                                    s.utc = DateTime.Parse(cas2,culture);
                                    ProcessSpot(s);
                                    break;
                                case 2: // Login request
                                    //SendString(Settings.Default.Login);
                                    break;
                                case 3: //Password request
                                    //SendString(Settings.Default.Password);
                                    break;

                            }
                            break;
                        }
                    }
                }
                pdata = pdata.Substring(posn + 1);
                posn = pdata.IndexOf('\n');
            }
        }

        // add date and convert to datetime
        // date added in order to minimize difference from actual time (midnight problem)
        private DateTime UtcToDatetime(string utc)
        {
            DateTime retval;
            utc = utc.Insert(utc.Length - 2, ":");
            DateTime[] dts = new DateTime[3];
            retval = dts[0] = DateTime.Parse(utc, culture);    // adds actual date
            TimeSpan day = new TimeSpan(24,0,0);    // one day
            dts[1] = dts[0] + day; // one day more
            dts[2] = dts[0] - day; // one day less
            DateTime now = DateTime.Now.ToUniversalTime();
            TimeSpan span = (dts[0] - now).Duration();
            for (int i = 1; i < 3; i++) // find datetime with minimum difference from current time
            {
                TimeSpan span2 = (dts[i] - now).Duration();
                if (span2 < span)
                {
                    retval = dts[i];
                    span = span2;
                }
            }
            return retval;
        }

        /// <summary>
        /// Send text string to DX cluster socket
        /// </summary>
        /// <param name="s"></param>
        private void SendString(string s)
        {
            try
            {
                if (soc.Connected)
                {
                    byte[] buf = new byte[s.Length + 2];

                    for (int i = 0; i < s.Length; i++)
                        buf[i] = (byte)s[i];

                    buf[s.Length] = 13;
                    buf[s.Length + 1] = 10;

                    soc.Send(buf);
                }
            }
            catch { }
        }


        /// <summary>
        /// add band and mode info to spot, call filter, add to datagrid
        /// </summary>
        /// <param name="s"></param>
        private void ProcessSpot(spot s)
        {
            loginDone = true;    // no more need to look for

            s.mode = "???";
            s.band = "??";
            foreach (DataRow br in bandPlanTable.Rows)
            {
                if ((s.mhz >= (double)br["mhz_from"]) && (s.mhz <= (double)br["mhz_to"]))
                {
                    s.band = (string)br["band"];
                    s.mode = (string)br["mode"];
                    break;
                }
            }

            DataRow dr = rawSpotTable.NewRow();
            dr["utc"] = s.utc;
            dr["call"] = s.call;
            dr["spotter"] = s.spotter;
            dr["mhz"] = s.mhz;
            dr["mode"] = s.mode;
            dr["band"] = s.band;

            while (rawSpotTable.Rows.Count > maxspots)
                rawSpotTable.Rows.RemoveAt(rawSpotTable.Rows.Count - 1);    // delete oldest spot when backlog too big
            rawSpotTable.Rows.InsertAt(dr,0);


            if (sfilter.TestMatch(s, ref alarmActive, out s.cont))
            {
                DataRow dr1 = filterSpotTable.NewRow();
                dr1["utc"] = s.utc;
                dr1["call"] = s.call;
                dr1["spotter"] = s.spotter;
                dr1["mhz"] = s.mhz;
                dr1["mode"] = s.mode;
                dr1["band"] = s.band;
                try
                {
                    filterSpotTable.Rows.InsertAt(dr1, 0);
                    if (dataGridView1.Rows.Count > 0)
                        dataGridView1.CurrentCell = dataGridView1.FirstDisplayedCell = dataGridView1.Rows[0].Cells[0];
                }
                catch (Exception ex)
                { }   // spot already exists (conflict on primary key), don't add it to table
            }
            dr["cont"] = s.cont;

            if (bm != null)
                if (bm.NewSpot(s) == true)
                    bm.Refresh();
        }

        private void rawDataToolStripMenuItem_Click(object sender, EventArgs e)
        {
            RawSpotsDlg dlg = new RawSpotsDlg(this);
            dlg.ShowDialog();
        }

        private void filterToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FiltersDlg dlg = new FiltersDlg();
            dlg.ShowDialog();
            sfilter.SaveFilters(filterFile);
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            Settings.Default.Width = Width;
            Settings.Default.Height = Height;
            Settings.Default.BMposn = bm.Location;
            Settings.Default.BMsize = bm.Size;
            Settings.Default.BMauto = bandMapAutoToolStripMenuItem.Checked;

            tcvr.SaveSettings();

            Settings.Default.Save();
            SaveSpots();

            start = false;
            SendString(" ");
            while (connected);
        }

        private void terminalWindowToolStripMenuItem_Click(object sender, EventArgs e)
        {
            tw.Show();
            tw.Focus();
        }

        [DllImport("user32.dll")]
        private static extern bool SetForegroundWindow(IntPtr hWnd);

        // every 500 msec
        private void timer1_Tick(object sender, EventArgs e)
        {
            bool newData = false;

            lock(this)
            {
                if (socketData.Length > 0)
                {
                    socketData = socketData.Replace('\a', ' ');
                    tw.txtRx.Text += socketData;
                    pdata += socketData;
                    socketData = string.Empty;
                    liveCtr = 600; // 5 minutes timeout reset when data received
                    newData = true;
                }
            }

            if (newData)
            {

                // it must be here - login and password req is not line terminated by <CR><LF>
                if (!loginDone)
                {
                    if (reLogin.IsMatch(pdata) || reCall.IsMatch(pdata) || reCallsign.IsMatch(pdata))
                    {
                        SendString(Settings.Default.Login);
                        pdata = "";
                    }
                    else if (rePassword.IsMatch(pdata))
                    {
                        SendString(Settings.Default.Password);
                        pdata = "";
                    }
                }
                while (tw.txtRx.Text.Length > 10000)
                {
                    tw.txtRx.Text = tw.txtRx.Text.Substring(500);
                }
                tw.txtRx.SelectionStart = tw.txtRx.Text.Length;
                tw.txtRx.SelectionLength = 0;
                tw.txtRx.ScrollToCaret();
                tw.txtRx.Refresh();
            }

            if (alarmActive)
            {
                System.Media.SystemSounds.Beep.Play();
                WindowState = FormWindowState.Normal;
                SetForegroundWindow(this.Handle);
            }

            ParseClusterText();

            toolStripStatusLabel1.Text = connected ? "Connected to " + Settings.Default.Server : "Disconnected" ;

            if (--bmCounter <= 0 )  // 1 minute = 120 x 500 msec
            {
                bmCounter = bmrefresh;
                if (bm != null)
                {
                    if (bm.CleanupSpots())
                        bm.Refresh();
                }
            }

            if ((bmCounter % 10 == 9) && Properties.Settings.Default.Autoconnect && !suspend && !connected) // every 5 sec
            {
                ConnectCluster();
            }

            if (connected)
            {
                if (liveCtr-- <= 0)
                {
                    liveCtr = 600;  // 5 minutes
                    SendString(" "); // send empty data to cluster when no data received for some time
                }
            }
            else
            {
                liveCtr = 120; // 1 minute
                loginDone = false;
            }
        }


        private void ackAlarmToolStripMenuItem_Click(object sender, EventArgs e)
        {
            alarmActive = false;
        }

        private void bandMapToolStripMenuItem_Click(object sender, EventArgs e)
        {
            bm.band = Settings.Default.BMband;
            if (Screen.FromControl(this).Bounds.Contains(Settings.Default.BMposn))
                bm.Location = Settings.Default.BMposn; 
            //bm.Location = new Point(Location.X + Width + 10, Location.Y);
            bm.Size = Settings.Default.BMsize;
            bm.Show();
            bm.Focus();
        }

        public void SaveBandMapSetting()
        {
            Settings.Default.BMband = bm.band;
        }

        public List<DxBand> GetBands()
        {
            List<DxBand> list = new List<DxBand>();

            foreach (DataRow dr in bandPlanTable.Rows)
            {
                string sband = (string)dr["Band"];
                bool found = false;
                foreach (DxBand band in list)
                {
                    if (band.name == sband)
                    {
                        found = true;
                        double f1 = (double)dr["MHz_from"];
                        double f2 = (double)dr["MHz_to"];
                        if (f1 < band.freq1)
                            band.freq1 = f1;
                        if (f2 > band.freq2)
                            band.freq2 = f2;
                    }
                }
                if (!found)
                {
                    DxBand newBand = new DxBand();
                    newBand.name = sband;
                    newBand.freq1 = (double)dr["MHz_from"];
                    newBand.freq2 = (double)dr["MHz_to"];
                    list.Add(newBand);
                }
            }
            return list;
        }

        /// <summary>
        /// Get list of all possible modes from bandplan table
        /// </summary>
        /// <returns></returns>
        public List<string> GetModes()
        {
            List<string> modes = new List<string>();
            foreach (DataRow dr in bandPlanTable.Rows)
            {
                string mode = dr["mode"] as string;
                mode = mode.Trim();
                bool found = false;
                foreach (string m in modes)
                {
                    if (m == mode)
                    {
                        found = true;
                        break;
                    }
                }
                if (!found)
                    modes.Add(mode);
            }
            return modes;
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MessageBox.Show("DX Watcher ver. " + Application.ProductVersion, "About", MessageBoxButtons.OK);
        }

        private void allToolStripMenuItem_Click(object sender, EventArgs e)
        {
            rawSpotTable.Rows.Clear();
            UpdateFilteredSpots();
        }

        private void dayOldToolStripMenuItem_Click(object sender, EventArgs e)
        {
            RemoveObsoleteSpots();
            UpdateFilteredSpots();
        }

        private void transceiverToolStripMenuItem_Click(object sender, EventArgs e)
        {
            tcvr.Show();
            tcvr.Focus();
        }

        private void bandMapAutoToolStripMenuItem_Click(object sender, EventArgs e)
        {
            bandMapAutoToolStripMenuItem.Checked = ! bandMapAutoToolStripMenuItem.Checked;
        }
    }

    public struct spot
    {
        public DateTime utc;
        public double mhz;
        public string call;
        public string band;
        public string mode;
        public string spotter;
        public string cont;

        public override string ToString()
        {
            return call + " " + band + " " + mode;
        }
    }

    public class DxFilter
    {
        public Regex re = null;  // regular expression filter
        public string band;
        public string mode;
        public bool alarm = false;

        public DxFilter(string expr, string band, string mode, bool alarm)
        {
            if ((expr != null) && (expr.Length > 0))
                re = new Regex(expr);
            this.band = band;
            this.mode = mode.ToUpper();
            this.alarm = alarm;
        }

        public bool TestRule(spot s, ref bool alarmActive)
        {
            if ((re != null) && !re.IsMatch(s.call))
                    return false;

            if ((band != null) && (band.Length > 0) && (s.band != band))
                return false;

            if ((mode != null) && (mode.Length > 0) && (mode != s.mode.ToUpper()))
                return false;

            if (alarm)
                alarmActive = true;
            return true;
        }
    }

    

    public class DxBand
    {
        public double freq1 = Double.MaxValue;
        public double freq2 = Double.MinValue;
        public string name = "";

        public override string ToString()
        {
            return name;
        }
    }
}
