﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Net.Sockets;

namespace DxWatcher
{
    public partial class TelnetWindow : Form
    {
        const int historySize = 10;
        MainForm parent;
        string[] hist = new string[historySize];    // history buffer [0]=most recent
        int histCount = 0;
        int histPtr = -1;


        public TelnetWindow(MainForm parent)
        {
            this.parent = parent;
            InitializeComponent();
        }

        private void TelnetWindow_FormClosing(object sender, FormClosingEventArgs e)
        {
            this.Hide();
            e.Cancel = true;
        }

        private void txTX_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyValue == 13)
            {
                HistoryAdd(txTX.Text);
                byte[] buffer = new byte[txTX.Text.Length + 2];
                for (int i = 0; i<txTX.Text.Length; i++)
                {
                    buffer[i] = (byte)txTX.Text[i];
                }
                buffer[txTX.Text.Length] = 13;
                buffer[txTX.Text.Length+1] = 10;

                if ((parent.soc != null) &&(parent.soc.Connected))
                    parent.soc.Send(buffer);
                txTX.Text = "";
            }
            else if (e.KeyValue == 38)
            {
                txTX.Text = HistoryGetUp();
            }
            else if (e.KeyValue == 40)
            {
                txTX.Text = HistoryGetDown();
            }
        }

        // save latest sent line
        void HistoryAdd(string line)
        {
            // empty lines not inserted into buffer
            if (line.Trim().Length == 0)
                return;

            if (histCount >= historySize)
                histCount = historySize - 1; // forget the oldest
            for (int i = histCount-1; i >= 0; i--)
            {
                hist[i + 1] = hist[i];  // shift the buffer 
            }
            hist[0] = line;
            histCount++;
            histPtr = -1;
        }

        // go to older history
        string HistoryGetUp()
        {
            histPtr++;
            if (histPtr >= histCount)
                histPtr = histCount - 1;
            if (histPtr < 0)
                return "";  // history empty
            return hist[histPtr];
        }

        // go to more recent history
        string HistoryGetDown()
        {
            histPtr--;
            if (histPtr < 0)
            {
                histPtr = -1;
                return "";  // at begin
            }
            return hist[histPtr];
        }

        private void TelnetWindow_Activated(object sender, EventArgs e)
        {
            txtRx.SelectionStart = txtRx.Text.Length;
            txtRx.SelectionLength = 0;
            txtRx.ScrollToCaret();
            txtRx.Refresh();
            txTX.Focus();
        }

        private void txtRx_KeyPress(object sender, KeyPressEventArgs e)
        {
            txTX.Focus();
            txTX.Text += e.KeyChar;
            txTX.SelectionStart = txTX.Text.Length;
        }




    }
}
